#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov  9 12:43:21 2020

@author: pharveycollard
"""

#%% Import modules
import numpy as np
import matplotlib.pyplot as plt, matplotlib

import os
import glob

import qcodes as qc
import qcodes.data.data_set as data_set
from qcodes.plots.qcmatplotlib import MatPlot as QcPlt
from scipy.optimize import curve_fit

#%% Matplotlib rc config 
# plt.rcdefaults() # reset rc defaluts

FONT_SIZE = 7
TITLE_SIZE = 7
LW = 0.75
# set subscript font size
matplotlib.mathtext.SHRINK_FACTOR = 0.8

FIG_WIDTH = 8 # cm
FIG_HEIGHT = 8 # cm
fig_size_list = [FIG_WIDTH/2.51, FIG_HEIGHT/2.51] # in inches
ax_position_tup = (1.4/FIG_WIDTH, 0.9/FIG_HEIGHT, 3.2/FIG_WIDTH, 3.2/FIG_HEIGHT) # relative units

plt.rcParams['font.sans-serif'] = ['Helvetica Neue',
                                   'Helvetica',
                                   'DejaVu Sans',
                                   'Bitstream Vera Sans',
                                   'Computer Modern Sans Serif',
                                   'Lucida Grande',
                                   'Verdana',
                                   'Geneva',
                                   'Lucid',
                                   'Arial',
                                   'Helvetica',
                                   'Avant Garde',
                                   'sans-serif']

# plt.rcParams['mathtext.fontset'] = 'dejavusans'
# plt.rcParams['mathtext.fontset'] = 'stixsans'
plt.rcParams['mathtext.fontset'] = 'custom'
plt.rcParams['mathtext.it'] = 'Arial:italic' # closest to helvetica that works...
plt.rcParams['mathtext.rm'] = 'Helvetica Neue' # works

plt.rc('font', size=FONT_SIZE)          # controls default text sizes
plt.rc('axes', labelsize=FONT_SIZE)     # fontsize of the x and y labels
plt.rc('axes', titlesize=TITLE_SIZE)    # fontsize of the axes title
plt.rc('xtick', labelsize=FONT_SIZE)    # fontsize of the tick labels
plt.rc('ytick', labelsize=FONT_SIZE)    # fontsize of the tick labels
plt.rc('legend', fontsize=FONT_SIZE, title_fontsize=None)    # legend fontsize
plt.rc('figure', titlesize=TITLE_SIZE)  # fontsize of the figure title

plt.rc('axes', linewidth=LW)
plt.rc('lines', linewidth=LW)

TL = 2.5
plt.rc('xtick.major', size=TL)           # tick length (default 3.5)
plt.rc('xtick.minor', size=TL/1.75)      # tick length (default 2)
plt.rc('ytick.major', size=TL)           # tick length (default 3.5)
plt.rc('ytick.minor', size=TL/1.75)      # tick length (default 2)

pcolor_opts = {'shading': 'nearest',
               'rasterized': True, }  # pass as **pcolor_opts

plt.rcParams['legend.frameon'] = True

plt.rcParams['figure.dpi'] = 129.0 # adjust to match display
plt.rcParams['figure.figsize'] = fig_size_list # in inches

plt.rcParams['savefig.dpi'] = 300
plt.rcParams['savefig.transparent'] = True

plt.rcParams['pdf.fonttype'] = 3 # 42 (TrueType) to edit PDF ext editor. Default 3 is more compact.

def set_plot_area(ah):
    ah.set_position(ax_position_tup)  # relative units

def make_colorbar_top(ah, ph, label='', clims=(None, None)):
    """ Generates a colorbar at the outer top right corner of the plot. 
    
    Clims must be set in this function, otherwise the colorbar resets itself. Ex.:
    ch, _, _ = make_colorbar_top(ah, ph, 'label')
    
    Use set_plot_area(ah) just before this function.
    
    ah: Axis object
    ph: Plot object
    label: string
    
    """
    axins1 = ah.inset_axes( (0.5, 1.05, .5, .05) ) # relative units
    # ph.set_cmap(plt.cm.YlGnBu_r)
    ph.set_clim(*clims)
    ch = ah.get_figure().colorbar(ph, cax=axins1, orientation='horizontal', shrink=0.5)
    axins1.xaxis.set_ticks_position('top')
    th = axins1.text(-0.1, 2.0, label, 
                     horizontalalignment='right', 
                     transform=axins1.transAxes) # set later with th.set_text('')
    return ch, axins1, th


#%% Functions
def find_closest(array, value, axis=None):
    """ Finds the closest elements in array to value along an axis.

    Parameters
    ----------
    array : ndarray, list
        A 1D or 2D array or list.
    value : scalar
        A scalar value to compare.
    axis : None, 0, 1, optional
        Recommended to provide one for 2D array. The default is None.

    Returns
    -------
    out_array : ndarray, one less dimention than array
        Contains the closest values along the axis specified. Not necessarily 
        along the same splice.
    idx : scalar or ndarray
        The indices of the closest elements along axis.

    """
    array = np.asarray(array)
    idx = np.argmin(np.abs(array-value), axis=axis)
    out_array = np.take(array, idx, axis=axis)
    if np.size(idx) > 1:
        out_array = np.diagonal(out_array)
    return out_array, idx

def fit_doubletanh(xdata, ydata, handle_bad_fit=False):
    """ Fits the data using a symmetric double hyperbolic tangent function to 
    find the zero-detuning point of a dispersive shift vs detuning dataset.

    """
    # Normalization and initial values
    x_scale = 10**np.floor(np.log10(np.max(np.abs(xdata))))
    xdata = xdata/x_scale
    y_scale = np.max(np.abs(ydata))
    ydata = ydata/y_scale
    
    x_range = np.max(xdata)-np.min(xdata)
    y_range = np.max(ydata)-np.min(ydata)
    x1_ini = np.min(xdata)+x_range/3
    x2_ini = np.max(xdata)-x_range/3
    x1_min = np.min(xdata)-x_range/4
    x1_max = np.max(xdata)-x_range/4
    x2_min = np.min(xdata)+x_range/4
    x2_max = np.max(xdata)+x_range/4
    
    def doubletanh(x, a, a0, w, x1, x2):
        y = a*(np.tanh(-(x-x1)/w)+1+np.tanh((x-x2)/w)+1)/2+a0
        return y
    
    # params: a,        a0,         w,          x1,     x2
    p_min = [y_range/3, 0,          x_range/10, x1_min, x2_min]
    p_ini = [y_range,   0,          x_range/4,  x1_ini, x2_ini]
    p_max = [y_range,   1-y_range/3,x_range/2,  x1_max, x2_max]
    
    try:
        popt, pcov = curve_fit(doubletanh, xdata, ydata, p0=p_ini, bounds=(p_min,p_max))
        perr = np.sqrt(np.diag(pcov)) # std dev, 2*perr is 95% conf int
    except RuntimeError as err:
        if handle_bad_fit == True:
            print(*err.args, '\n'+'Returning initial fit parameters instead!')
            popt = p_ini
            perr = np.tile(float('nan'), np.size(popt))
        else:
            raise
    
    # # Plot (for debugging)
    # plt.close(2)
    # plt.figure(2)
    # plt.plot(xdata, ydata, 'o')
    # plt.plot(xdata, doubletanh(xdata, *popt))
    # plt.xlabel('xdata/x_scale')
    # plt.ylabel('ydata/y_scale')
    # plt.show()
    
    # Normalize back
    for i in [0, 1]:
        popt[i] = popt[i]*y_scale
        perr[i] = perr[i]*y_scale
    for i in [2, 3, 4]:
        popt[i] = popt[i]*x_scale
        perr[i] = perr[i]*x_scale
    
    return popt, perr, doubletanh

def fit_doubletanh_hump(xdata, ydata, handle_bad_fit=False):
    """ Fits the data using a symmetric double hyperbolic tangent function to 
    find the zero-detuning point of a dispersive shift vs detuning dataset.
    This version adds a hump to account for when the B field puts a hump inside 
    the fit cut.
    
    Careful: The hump algorithm needs to see both sides of the unshifted 
    resonator signal to work well!

    """
    # Normalization and initial values
    x_scale = 10**np.floor(np.log10(np.max(np.abs(xdata))))
    xdata = xdata/x_scale
    y_scale = np.max(np.abs(ydata))
    ydata = ydata/y_scale
    
    x_range = np.max(xdata)-np.min(xdata)
    y_range = np.max(ydata)-np.min(ydata)
    x1_ini = np.min(xdata)+x_range/5
    x2_ini = np.max(xdata)-x_range/5
    x1_min = np.min(xdata)-x_range/8
    x1_max = np.max(xdata)-x_range/3
    x2_min = np.min(xdata)+x_range/3
    x2_max = np.max(xdata)+x_range/8
    
    def doubletanh_hump(x, a, a0, w, x1, x2, has, hx1s, hx2s):
        def hump(x, a, a0, w, x1, x2):
            """ This is a doubletanh_peak. """
            y = a*(np.tanh((x-x1)/w)-1+np.tanh(-(x-x2)/w)+1)/2+a0
            return y
        y = ( a*(np.tanh(-(x-x1)/w)+1+np.tanh((x-x2)/w)+1)/2+a0 + 
              hump(x, a*has, 0, w/2.5, x1+(x2-x1)*hx1s, x2-(x2-x1)*hx2s) )
        return y
    
    # params: a,            a0,         w,          x1,     x2,     has,    hx1s,   hx2s
    p_min = [y_range*0.7,   0,          x_range/12, x1_min, x2_min, 0,      0.0,    0.0]
    p_ini = [y_range,       0,          x_range/8,  x1_ini, x2_ini, 0.2,    0.4,    0.1]
    p_max = [y_range,       1-y_range/4,x_range/5,  x1_max, x2_max, 0.5,    0.7,    0.5]
    
    try:
        popt, pcov = curve_fit(doubletanh_hump, xdata, ydata, p0=p_ini, bounds=(p_min,p_max))
        perr = np.sqrt(np.diag(pcov)) # std dev, 2*perr is 95% conf int
    except RuntimeError as err:
        if handle_bad_fit == True:
            print(*err.args, '\n'+'Returning initial fit parameters instead!')
            popt = p_ini
            perr = np.tile(float('nan'), np.size(popt))
        else:
            raise
    
    # # Plot (for debugging)
    # plt.figure()
    # plt.plot(xdata, ydata, 'o')
    # plt.plot(xdata, doubletanh_hump(xdata, *popt))
    # plt.xlabel('xdata/x_scale')
    # plt.ylabel('ydata/y_scale')
    # plt.show()
    
    # Normalize back
    for i in [0, 1]:
        popt[i] = popt[i]*y_scale
        perr[i] = perr[i]*y_scale
    for i in [2, 3, 4]:
        popt[i] = popt[i]*x_scale
        perr[i] = perr[i]*x_scale
    
    return popt, perr, doubletanh_hump

def fit_doubletanh_peak(xdata, ydata, handle_bad_fit=False):
    """ Fits the data using a symmetric double hyperbolic tangent function to 
    find the zero-detuning point of peak vs detuning dataset.

    """

    # Normalization and initial values
    x_scale = 10**np.floor(np.log10(np.max(np.abs(xdata))))
    xdata = xdata/x_scale
    y_scale = np.max(np.abs(ydata))
    ydata = ydata/y_scale
    
    x_range = np.max(xdata)-np.min(xdata)
    y_range = np.max(ydata)-np.min(ydata)
    x1_ini = np.min(xdata)+x_range/3
    x2_ini = np.max(xdata)-x_range/3
    x1_min = np.min(xdata)-x_range/4
    x1_max = np.max(xdata)-x_range/4
    x2_min = np.min(xdata)+x_range/4
    x2_max = np.max(xdata)+x_range/4
    
    def doubletanh_peak(x, a, a0, w, x1, x2):
        """ This is a double hyperbolic tangent peak function. """
        y = a*(np.tanh((x-x1)/w)-1+np.tanh(-(x-x2)/w)+1)/2+a0
        return y
    
    # params: a,        a0,         w,          x1,     x2
    p_min = [y_range/3, 0,          x_range/10, x1_min, x2_min]
    p_ini = [y_range,   0,          x_range/4,  x1_ini, x2_ini]
    p_max = [y_range,   1,          x_range/2,  x1_max, x2_max]
    
    try:
        popt, pcov = curve_fit(doubletanh_peak, xdata, ydata, p0=p_ini, bounds=(p_min,p_max))
        perr = np.sqrt(np.diag(pcov)) # std dev, 2*perr is 95% conf int
    except RuntimeError as err:
        if handle_bad_fit == True:
            print(*err.args, '\n'+'Returning initial fit parameters instead!')
            popt = p_ini
            perr = np.tile(float('nan'), np.size(popt))
        else:
            raise
    
    # # Plot (for debugging)
    # plt.close(2)
    # plt.figure(2)
    # plt.plot(xdata, ydata, 'o')
    # plt.plot(xdata, doubletanh_peak(xdata, *popt))
    # plt.xlabel('xdata/x_scale')
    # plt.ylabel('ydata/y_scale')
    # plt.show()
    
    # Normalize back
    for i in [0, 1]:
        popt[i] = popt[i]*y_scale
        perr[i] = perr[i]*y_scale
    for i in [2, 3, 4]:
        popt[i] = popt[i]*x_scale
        perr[i] = perr[i]*x_scale
    
    return popt, perr, doubletanh_peak

def plot_center_fit(x1_data, y1_data, z1_data, 
                    x1_label, y1_label, z1_label, title1,
                    x2_data, y2_data, fit_f, popt, F_REF, x0):
    """ Plots the data and fit from fit_doubletanh.

    """
    fh, ah = plt.subplots(1,2, figsize=(6.4*2,4.8*1))
    
    plt.sca(ah[0])
    # indices[y, x]
    plt.pcolormesh(x1_data, y1_data, z1_data, shading='nearest')
    plt.colorbar(label=z1_label)
    plt.xlabel(x1_label)
    plt.ylabel(y1_label)
    plt.title(title1)
    xlim = list(plt.gca().get_xlim())
    ylim = list(plt.gca().get_ylim())
    plt.hlines(F_REF, xlim[0], xlim[1], color='k', linestyle='--')
    plt.vlines(x0, ylim[0], ylim[1], color='w', linestyle='--')
    
    plt.sca(ah[1])
    plt.plot(x2_data, y2_data, 'o')
    plt.plot(x2_data, fit_f(x2_data, *popt))
    plt.xlabel('xdata')
    plt.ylabel('zdata')
    
    plt.tight_layout()
    plt.show()

def fit_fano(xdata, ydata, EXPONENT=2, handle_bad_fit=False):
    """ Fits the data using a Fano resonance lineshape. 
    
    Use the EXPONENT=1 option to fit the transmission data directly to the 
    sqrt(lorentzian). In this case fwhm is still the lorentzian FWHM, i.e. the 
    full width at 0.71*max.

    """
    # Normalization and initial values
    x_scale = 10**np.floor(np.log10(np.max(np.abs(xdata))))
    xdata = xdata/x_scale
    y_scale = np.max(np.abs(ydata))
    ydata = ydata/y_scale
        
    x_range = np.max(xdata)-np.min(xdata)
    y_range = np.max(ydata)-np.min(ydata)
        
    idx = np.argmin(np.abs(ydata-np.max(ydata)))
    f0_ini = np.take(xdata, idx)
    f0_min = np.min(xdata)
    f0_max = np.max(xdata)
        
    def fano_peak(x, f0, fwhm, a, q, r):
        """
        Fano resonance peak function (if EXPONENT=2). Can be used for Lorentzian 
        fit if q->Inf and r=0.

        Parameters
        ----------
        x : vector
            Frequency data.
        f0 : scalar
            Resonance frequency.
        fwhm : scalar
            Full width at half maximum.
        a : scalar
            Amplitude.
        q : scalar > 1
            Fano factor magnitude. When q -> Inf, the functions reduces to a 
            Lorentzian lineshape.
        r : scalar [0, pi]
            Fano factor phase in rad. For r=0, the resonance peak has a higher 
            transmission on the high frequency side. For r=pi, it is on the low 
            frequency side.

        Returns
        -------
        y : vector
            Transmission data.

        """
        y = a*np.abs( ( (x-f0)/(q*np.exp(1j*r))+fwhm/2 )/( 1j*(x-f0)+fwhm/2 ) )**EXPONENT
        return y
    
    # params: f0,       fwhm,       a,          q,          r
    p_min = [f0_min,    0,          0,          1,          0]
    p_ini = [f0_ini,    x_range/5,  y_range,    10,         np.pi/2]
    p_max = [f0_max,    x_range,    y_range,    1e5,        np.pi]
    
    try:
        popt, pcov = curve_fit(fano_peak, xdata, ydata, p0=p_ini, bounds=(p_min,p_max))
        perr = np.sqrt(np.diag(pcov)) # std dev, 2*perr is 95% conf int
    except RuntimeError as err:
        if handle_bad_fit == True:
            print(*err.args, '\n'+'Returning initial fit parameters instead!')
            popt = p_ini
            perr = np.tile(float('nan'), np.size(popt))
        else:
            raise
    
    # # Plot (for debugging)
    # plt.close(2)
    # plt.figure(2)
    # plt.plot(xdata, ydata, 'o')
    # plt.plot(xdata, fano_peak(xdata, *popt))
    # plt.xlabel('xdata/x_scale')
    # plt.ylabel('ydata/y_scale')
    # plt.show()
    
    # Normalize back
    for i in [2]:
        popt[i] = popt[i]*y_scale
        perr[i] = perr[i]*y_scale
    for i in [0, 1]:
        popt[i] = popt[i]*x_scale
        perr[i] = perr[i]*x_scale
    
    return popt, perr, fano_peak

def fit_lorentzian_arb(xdata, ydata, EXPONENT=2, handle_bad_fit=False):
    """ Fits the data using a Lorentzian resonance lineshape. This version allows
    for an arbitrary offset of the data and a peak or dip.
    
    Use the EXPONENT=1 option to fit the transmission data directly to the 
    sqrt(lorentzian). In this case fwhm is still the lorentzian FWHM, i.e. the 
    full width at 0.71*max.

    """
    # Normalization and initial values
    x_scale = 10**np.floor(np.log10(np.max(np.abs(xdata))))
    xdata = xdata/x_scale
    y_scale = np.max(np.abs(ydata))
    ydata = ydata/y_scale
    
    x_range = np.max(xdata)-np.min(xdata)
    y_range = np.max(ydata)-np.min(ydata)
    
    if np.max(ydata)-np.mean(ydata) > np.mean(ydata)-np.min(ydata):
        # fit for a peak
        idx = np.argmin(np.abs(ydata-np.max(ydata)))
        a_ini = np.max(ydata) - np.mean(ydata)
    else:
        # fit for a dip
        idx = np.argmin(np.abs(ydata-np.min(ydata)))
        a_ini = np.min(ydata) - np.mean(ydata)

    f0_ini = np.take(xdata, idx)
    f0_min = np.min(xdata)
    f0_max = np.max(xdata)
    a0_ini = np.mean(ydata)
    a0_min = a0_ini - 2.53*np.std(ydata)*2 # FWHM = 2.53*STD
    a0_max = a0_ini + 2.53*np.std(ydata)*2 # FWHM = 2.53*STD
    
    def lorentzian_arb(x, f0, fwhm, a, a0):
        """
        Lorentz resonance peak function (if EXPONENT=2). Allows for an offset 
        from zero, and a peak or dip.

        Parameters
        ----------
        x : numpy.ndarray, vector
            Frequency data.
        f0 : scalar
            Resonance frequency.
        fwhm : scalar
            Full width at half maximum.
        a : scalar
            Amplitude (can be positive or negative).
        a0 : scalar
            Baseline offset away from resonance.

        Returns
        -------
        y : vector
            Transmission data.

        """
        y = a*np.abs( ( fwhm/2 )/( 1j*(x-f0)+fwhm/2 ) )**EXPONENT + a0
        return y
    
    # params: f0,       fwhm,       a,          a0
    p_min = [f0_min,    0,          -y_range,   a0_min]
    p_ini = [f0_ini,    x_range/10, a_ini,      a0_ini]
    p_max = [f0_max,    x_range,    y_range,    a0_max]
    
    try:
        popt, pcov = curve_fit(lorentzian_arb, xdata, ydata, p0=p_ini, bounds=(p_min,p_max))
        perr = np.sqrt(np.diag(pcov)) # std dev, 2*perr is 95% conf int
    except RuntimeError as err:
        if handle_bad_fit == True:
            print(*err.args, '\n'+'Returning initial fit parameters instead!')
            popt = p_ini
            perr = np.tile(float('nan'), np.size(popt))
        else:
            raise
    
    # # Plot (for debugging)
    # plt.close(2)
    # plt.figure(2)
    # plt.plot(xdata, ydata, 'o')
    # plt.plot(xdata, lorentzian_arb(xdata, *p_ini))
    # plt.plot(xdata, lorentzian_arb(xdata, *popt))
    # plt.xlabel('xdata/x_scale')
    # plt.ylabel('ydata/y_scale')
    # plt.show()
    
    # Normalize back
    for i in [2, 3]:
        popt[i] = popt[i]*y_scale
        perr[i] = perr[i]*y_scale
    for i in [0, 1]:
        popt[i] = popt[i]*x_scale
        perr[i] = perr[i]*x_scale
    
    return popt, perr, lorentzian_arb

def plot_fano_fit(x1_data, y1_data, 
                  x1_label, y1_label, title1,
                  fit_f, popt):
    """ 
    Plots the data and fit from fit_fano.
    
    """
    fh, ah = plt.subplots(1,1)
    
    plt.sca(ah)
    plt.plot(x1_data, y1_data, 'o')
    plt.plot(x1_data, fit_f(x1_data, *popt))

    plt.xlabel(x1_label)
    plt.ylabel(y1_label)
    plt.title(title1)
        
    plt.tight_layout()
    plt.show()

def plot_center_and_fano_fit(x1_data, y1_data, z1_data, x1_label, y1_label, z1_label, title1,
                             x2_data, y2_data, fit_f2, popt2, F_REF, x0, 
                             x3_data, y3_data, fit_f3, popt3):
    """ Plots the data and fit from fit_doubletanh and fit_fano together.

    """
    x1_label = 'RP2 voltage (mV)'
    y1_label = 'Frequency (Hz)'
    z1_label = r'|S21| (arb. u.)'
    
    fh = plt.figure(figsize=fig_size_list)
    gh = plt.GridSpec(2, 2)
    
    ah = fh.add_subplot(gh[:,0])
    plt.sca(ah)
    # indices[y, x]
    ph = plt.pcolormesh(x1_data, y1_data, z1_data, **pcolor_opts)
    # plt.colorbar(label=z1_label)
    make_colorbar_top(ah, ph, z1_label)
    plt.xlabel(x1_label)
    plt.ylabel(y1_label)
    # plt.title(title1, pad=16)
    xlim = list(plt.gca().get_xlim())
    ylim = list(plt.gca().get_ylim())
    plt.hlines(F_REF, xlim[0], xlim[1], color='k', linestyle='--')
    plt.vlines(x0, ylim[0], ylim[1], color='w', linestyle='--')
    
    ah = fh.add_subplot(gh[0,1])
    plt.sca(ah)
    plt.plot(x2_data, y2_data, 'o', markersize=LW)
    plt.plot(x2_data, fit_f2(x2_data, *popt2))
    plt.xlabel(x1_label)
    plt.ylabel(z1_label)
    ah.annotate('x0 = {:.4g} mV'.format(x0),
            xy=(0.5, 1), xycoords='axes fraction',
            xytext=(0, -20), textcoords='offset pixels',
            horizontalalignment='center', verticalalignment='top')
    
    ah = fh.add_subplot(gh[1,1])
    plt.sca(ah)
    plt.plot(x3_data, y3_data, 'o', markersize=LW)
    plt.plot(x3_data, fit_f3(x3_data, *popt3))
    plt.xlabel(y1_label)
    plt.ylabel(z1_label)
    ah.annotate('f0 = {:.5g} GHz,\nfwhm = {:.2g} MHz'.format(popt3[0]/1e9, popt3[1]/1e6),
            xy=(1, 1), xycoords='axes fraction',
            xytext=(-20, -20), textcoords='offset pixels',
            horizontalalignment='right', verticalalignment='top')
    
    fh.tight_layout()
    plt.show()
    plt.savefig(title1.replace('/', '_')+'.pdf')

#%% Find the data files

# path of directory containing the data, or current working directory
dirpath = os.getcwd()

# find all '.dat' files in subdirectories
filelist = glob.glob('*/*/*.dat') + glob.glob('*/*.dat') + glob.glob('*.dat')
filelist.sort()

# split directories and file names
pathlist, filenames = [], []
for i, path in enumerate(filelist):
    path, name = os.path.split(path)
    pathlist.append(path)
    filenames.append(name)
del name, path, i

#%% Import the data
io = qc.data.io.DiskIO(dirpath)
dslist = []

for i, path in enumerate(pathlist):
    # import qcodes dataset
    fullpath = os.path.normpath(os.path.join(dirpath,path)) # similar to 'dir/'.rstrip('/')
    ds = data_set.load_data(fullpath, io=io) # loads the GNUplot .dat and snapshop.json
    ds.metadata['station']['instruments']['acquisition_controller_fast']['parameters']['acquisition']['value'] = []
    # list of dataset parameters: paramlist = list(ds.arrays.keys())
    mdic = {}
    for key in ds.arrays.keys():
        mdic[key] = ds.arrays[key].ndarray
    # data from snapshot to append
    mdic['Bsper'] = ds.metadata['station']['instruments']['triton']['parameters']['Bspher']['value']
    # record the data in dslist
    gate_label = list(ds.arrays.keys())[1]
    ds.gate = getattr(ds, gate_label)
    dslist.append(ds)
del path, fullpath, key, mdic, i, ds

#%% plot
ds = dslist[0]

plt.close(1)
plt.figure(1)
# indices[y, x]
plt.pcolormesh(ds.gate.ndarray, ds.frequency.ndarray, ds.S21mag.ndarray, shading='nearest')
plt.colorbar(label=ds.S21mag.label)
plt.xlabel(ds.gate.label)
plt.ylabel(ds.frequency.label)
plt.show()


#%% Fit for center
# for i in range(0, len(dslist)):
# # for i in range(0, 1):

#     ds = dslist[i]
    
#     title = ds.metadata['scantime'][0:10] + '/' + os.path.split(ds.location)[1]
    
#     # Fit for center
#     F_REF = 6.915e9
#     xdata = ds.gate.ndarray[1,:]
#     val, idx = find_closest(ds.frequency.ndarray, F_REF)
#     ydata = ds.S21mag.ndarray[idx,:]
    
#     popt, perr, doubletanh = fit_doubletanh(xdata, ydata)
#     x0 = np.mean(popt[3:5])
    
#     # Fano fit on center linecut
#     xdata2 = ds.frequency.ndarray
#     val, idx = find_closest(ds.gate.ndarray[1,:], x0)
#     ydata2 = ds.S21mag.ndarray[:,idx]
    
#     popt2, perr2, fano_peak = fit_fano(xdata2, ydata2, EXPONENT=1)
    
#     plot_center_and_fano_fit(ds.gate.ndarray, ds.frequency.ndarray, ds.S21mag.ndarray, 
#                              ds.gate.label, ds.frequency.label, ds.S21mag.label, title,
#                              xdata, ydata, doubletanh, popt, F_REF, x0,
#                              xdata2, ydata2, fano_peak, popt2)


#%% Fit for center with doubletanh_hump and lorentzian_arb
# import pdb; pdb.set_trace()
plt.close('all')
for i in range(0, len(dslist)):
# for i in range(0, 1):

    ds = dslist[i]
    
    title = ds.metadata['scantime'][0:10] + '/' + os.path.split(ds.location)[1]
    
    # Fit for center
    F_REF = 6.915e9
    xdata = ds.gate.ndarray[1,:]
    val, idx = find_closest(ds.frequency.ndarray, F_REF)
    ydata = ds.S21mag.ndarray[idx,:]
    
    popt, perr, doubletanh = fit_doubletanh_hump(xdata, ydata, handle_bad_fit=True)
    x0 = np.mean(popt[3:5])
    
    # Lorentzian fit on center linecut
    xdata2 = ds.frequency.ndarray
    val, idx = find_closest(ds.gate.ndarray[1,:], x0)
    ydata2 = ds.S21mag.ndarray[:,idx]
    
    popt2, perr2, fano_peak = fit_fano(xdata2, ydata2, EXPONENT=1)
    
    plot_center_and_fano_fit(ds.gate.ndarray, ds.frequency.ndarray, ds.S21mag.ndarray, 
                             ds.gate.label, ds.frequency.label, ds.S21mag.label, title,
                             xdata, ydata, doubletanh, popt, F_REF, x0,
                             xdata2, ydata2, fano_peak, popt2)
